/* SPDX-License-Identifier: GPL-2.0 */
#ifndef __DT_TLV320AIC31XX_MICBIAS_H
#define __DT_TLV320AIC31XX_MICBIAS_H

#define MICBIAS_2_0V		1
#define MICBIAS_2_5V		2
#define MICBIAS_AVDDV		3

#endif /* __DT_TLV320AIC31XX_MICBIAS_H */
