/* SPDX-License-Identifier: GPL-2.0 */
/*
 * This header provides clock numbers for the Ingenic OST DT binding.
 */

#ifndef __DT_BINDINGS_CLOCK_INGENIC_OST_H__
#define __DT_BINDINGS_CLOCK_INGENIC_OST_H__

#define OST_CLK_PERCPU_TIMER	1
#define OST_CLK_GLOBAL_TIMER	0
#define OST_CLK_PERCPU_TIMER0	1
#define OST_CLK_PERCPU_TIMER1	2
#define OST_CLK_PERCPU_TIMER2	3
#define OST_CLK_PERCPU_TIMER3	4

#define OST_CLK_EVENT_TIMER		1

#define OST_CLK_EVENT_TIMER0	0
#define OST_CLK_EVENT_TIMER1	1
#define OST_CLK_EVENT_TIMER2	2
#define OST_CLK_EVENT_TIMER3	3
#define OST_CLK_EVENT_TIMER4	4
#define OST_CLK_EVENT_TIMER5	5
#define OST_CLK_EVENT_TIMER6	6
#define OST_CLK_EVENT_TIMER7	7
#define OST_CLK_EVENT_TIMER8	8
#define OST_CLK_EVENT_TIMER9	9
#define OST_CLK_EVENT_TIMER10	10
#define OST_CLK_EVENT_TIMER11	11
#define OST_CLK_EVENT_TIMER12	12
#define OST_CLK_EVENT_TIMER13	13
#define OST_CLK_EVENT_TIMER14	14
#define OST_CLK_EVENT_TIMER15	15

#endif /* __DT_BINDINGS_CLOCK_INGENIC_OST_H__ */
