/* SPDX-License-Identifier: GPL-2.0 */
#ifndef __AUDIO_JACK_EVENTS_H
#define __AUDIO_JACK_EVENTS_H

#define JACK_HEADPHONE		1
#define JACK_MICROPHONE		2
#define JACK_LINEOUT		3
#define JACK_LINEIN		4

#endif /* __AUDIO_JACK_EVENTS_H */
