/* This file is auto generated, version 1 */
/* SMP PREEMPT */
#define UTS_MACHINE "x86_64"
#define UTS_VERSION "#1 SMP PREEMPT Sat, 26 Mar 2022 18:04:40 +0000"
#define LINUX_COMPILE_BY "linux-ublinux"
#define LINUX_COMPILE_HOST "ublinux"
#define LINUX_COMPILER "gcc (GCC) 11.1.0, GNU ld (GNU Binutils) 2.36.1"
