/* SPDX-License-Identifier: GPL-2.0 */
/*
 * Copyright (c) 2018, The Linux Foundation. All rights reserved.
 */

#ifndef _DT_BINDINGS_CLK_SDM_LPASS_SDM845_H
#define _DT_BINDINGS_CLK_SDM_LPASS_SDM845_H

#define LPASS_Q6SS_AHBM_AON_CLK				0
#define LPASS_Q6SS_AHBS_AON_CLK				1
#define LPASS_QDSP6SS_XO_CLK				2
#define LPASS_QDSP6SS_SLEEP_CLK				3
#define LPASS_QDSP6SS_CORE_CLK				4

#endif
