/* SPDX-License-Identifier: GPL-2.0+ */
/*
 *  IPv6 IOAM
 *
 *  Author:
 *  Justin Iurman <justin.iurman@uliege.be>
 */
#ifndef _LINUX_IOAM6_H
#define _LINUX_IOAM6_H

#include <uapi/linux/ioam6.h>

#endif /* _LINUX_IOAM6_H */
